; ##############################################################################
; ##############################################################################
; ##############################################################################
; WDTCSR  Watchdog Timer Control Register				
; WDIF WDIE WDP3 WDCE WDE WDP2 WDP1 WDP0				
; WDP3 WDP2 WDP1 WDP0 Time								
;	0 	0 	0 	0 	2K 	(2048) 	cycles 16 ms			
;	0 	0 	0 	1 	4K 	(4096) 	cycles 32 ms			
;	0 	0 	1 	0 	8K 	(8192) 	cycles 64 ms			
;	0 	0 	1 	1 	16K (16384) cycles 0.125 s			
;	0 	1 	0 	0 	32K (32768) cycles 0.25 s			
; 	0 	1 	0 	1 	64K (65536) cycles 0.5 s			
; *	0 	1 	1 	0 	128K (131072)cycles 1.0 s			
;	0 	1 	1 	1 	256K (262144)cycles 2.0 s			
;	1 	0 	0 	0 	512K (524288)cycles 4.0 s			
;	1 	0 	0 	1 	1024K(1048576)cycles 8.0 s			
; Wenn WDIE=1 dann wird zuerst die WDR-ISR aufgerufen 	
; und dann das System neu gestartet						
WDR_INITIALISIERUNG:
	WDR
	LDS TEMP, WDTCSR
	ori TEMP, (1<<WDCE) | (1<<WDE)
	STS WDTCSR, TEMP
	ldi temp, (1<<WDE)| (1<<WDIE)|(0<<WDP3)|(1<<WDP2)|(1<<WDP1)|(0<<WDP0)
	STS WDTCSR, TEMP
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Diese Routine wird bei WDR-Zeitberschreitung aufgerufen, hier knnte man		
; sich z.B. den Fehler anzeigen lasen, oder DATEN in den EEPROM schreiben		
WDT:
	CLI											; INTERRUPTS sperren	
	ldi temp9,10
WDT_s:
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	cbi (LED_GELB_PORT) ,(LED_GELB_PIN_NR)		; GELB an	
	call wait_100ms
	sbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  aus	
	sbi (LED_GELB_PORT)  ,(LED_GELB_PIN_NR)		; GELB aus	
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN aus	
	call wait_100ms
	dec temp9
	brne WDT_s
WDT_s1:
	rjmp WDT_s1
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
SRAM_CLEAR:
	ldi ZL, LOW (SRAM_START)
	ldi ZH, HIGH(SRAM_START)
	ldi temp,0
SRAM_CLEAR_w:
	ST z+,temp
	cpi ZH, HIGH(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	cpi ZL, LOW(RAMEND-2)	; Vergleich?		
	brlo SRAM_CLEAR_w		; kleiner => SPRUNG	
	breq SRAM_CLEAR_w		; gleich  => SPRUNG	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
STARTWERTE:
	; NULL						
	clr NULL
	; EINS						
	ldi temp,1
	mov EINS,temp
	; VOLL						
	ldi temp,255
	mov VOLL,temp
	; VERSION					
	ldi temp,(VERSION)
	STS(adr_VERSION),temp
	; SRAM CHECK				
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
	; DATUM / ZEIT				
	ldi temp,0
	STS(adr_ss),temp
	ldi temp,0
	STS(adr_mm),temp
	ldi temp,0
	STS(adr_hh),temp
	ldi temp,13
	STS(adr_TAG),temp
	ldi temp,6
	STS(adr_MONAT),temp
	ldi temp,19
	STS(adr_JAHR),temp
	ldi temp,4
	STS(adr_WOCHENTAG),temp
;-------------------------------------------------------------------------------
	ldi ZL,low (adr_SENDER_TASTE200)
	ldi ZH,high(adr_SENDER_TASTE200)
	ldi temp9,8
	ldi temp,'0'
	ldi temp1, low(9440)	; Frequenz	
	ldi temp2,high(9440)
	ldi temp3,MODUS_FM
	ldi temp8,' '	; NAME			
STARTWERTE_s:
	ST Z+,temp8	;0 NAME		
	ST Z+,temp8	;1
	ST Z+,temp8	;2
	ST Z+,temp8	;3
	ST Z+,temp8	;4
	ST Z+,temp8	;5
	ST Z+,temp8	;6
	ST Z+,temp8	;7
	ST Z+,temp3	;8 Modus		
	ST Z+,temp1	;9 Frequenz	
	ST Z+,temp2	;10 Frequenz	
	ST Z+,NULL 	;11 Dummy		
	ST Z+,NULL 	;12
	ST Z+,NULL 	;13
	ST Z+,NULL 	;14
	ST Z+,NULL 	;15
	ST Z+,NULL 	;16
	ST Z+,NULL 	;17
	ST Z+,NULL 	;18
	ST Z+,NULL 	;19

	subi temp1, LOW (-10)	; Frequenz		
	sbci temp2, HIGH(-10)
	dec temp9
	brne STARTWERTE_s
;-------------------------------------------------------------------------------


ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; PORT CLEAR																	
PORT_CLEAR:
	ldi temp, 0b00000000	
	out DDRA, temp
	out DDRB, temp
	out DDRC, temp
	out DDRD, temp
	out PORTA,temp				
	out PORTB,temp				
	out PORTC,temp				
	out PORTD,temp	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_NEUSTART:
	CLI											; INTERRUPTS sperren	
	call TFT_CLEAR
	call TEXT_RESET
	LED_ROT_AN
	LED_GELB_AN
	LED_GRUEN_AN
	LED_STEREO_AN
	BEEP_AN
PROGRAMM_NEUSTART_r:
	rjmp PROGRAMM_NEUSTART_r
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_CHECK:
	; NULL		
	tst NULL
	brne PROGRAMM_CHECK_ERROR
	; EINS		
	mov temp,EINS
	cpi temp,1
	brne PROGRAMM_CHECK_ERROR
	; VOLL		
	mov temp,VOLL
	cpi temp,255
	brne PROGRAMM_CHECK_ERROR
	; SRAM CHECK
	LDS temp,(adr_SRAM_CHECK)
	cpi temp,'S'
	brne PROGRAMM_CHECK_ERROR
	ret
;-------------------------------------------------------------------------------
PROGRAMM_CHECK_ERROR:
	CLI											; INTERRUPTS sperren	
	call wait_100ms
	cbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN an	
	cbi (LED_GELB_PORT) ,(LED_GELB_PIN_NR)		; GELB an	
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  an	
	call wait_100ms
	sbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  aus	
	sbi (LED_GELB_PORT)  ,(LED_GELB_PIN_NR)		; GELB aus	
	sbi (LED_GRUEN_PORT),(LED_GRUEN_PIN_NR)		; GRN aus	
	rjmp PROGRAMM_CHECK_ERROR
; ##############################################################################
; ##############################################################################
; ##############################################################################
LED_INITIALISIERUNG:
	sbi (LED_GRUEN_DDR) ,(LED_GRUEN_PIN_NR)		; GRN		
	sbi (LED_GELB_DDR)  ,(LED_GELB_PIN_NR)		; GELB		
	sbi (LED_ROT_DDR)   ,(LED_ROT_PIN_NR)		; ROT		
	sbi (LED_GRUEN2_DDR),(LED_GRUEN2_PIN_NR)	; GRN2		
	sbi (LED_GELB2_DDR) ,(LED_GELB2_PIN_NR)		; GELB2		
	sbi (LED_ROT2_DDR)  ,(LED_ROT2_PIN_NR)		; ROT2		
	sbi (LED_STEREO_DDR),(LED_STEREO_PIN_NR)	; Stereo	
	sbi (BEEP_DDR)		,(BEEP_PIN_NR)			; BEEP		
;-------------------------------------------------------------------------------
	BEEP_AUS
	LED_STEREO_AN
	LED_GRUEN_AN
	LED_GELB_AN
	LED_ROT_AN
	LED_GRUEN2_AN
	LED_GELB2_AN
	LED_ROT2_AN
	call WAIT_500ms
	LED_STEREO_AUS
	LED_GRUEN_AUS
	LED_GELB_AUS
	LED_ROT_AUS
	LED_GRUEN2_AUS
	LED_GELB2_AUS
	LED_ROT2_AUS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
FAVORIT_LOAD_TEMP1:
	ldi temp,110
	add temp,temp1
	STS(adr_TASTE),temp
	jmp MENUE_110
; ##############################################################################
; ##############################################################################
; ##############################################################################
BEEP:
	BEEP_AN
	call WAIT_100ms
	BEEP_AUS
ret
; ##############################################################################
BEEP_LANG:
	BEEP_AN
	call WAIT_1s
	BEEP_AUS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PROGRAMM_STOP:
	WDR
	rjmp PROGRAMM_STOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Bsp 120 ---> 02:00 															
ZEITZAHL_ZEIT:
	ldi temp9,(-1)
ZEITZAHL_ZEIT_s:
	inc temp9
	subi temp1, BYTE1    (60)
	sbci temp2, BYTE2    (60)
	brcc ZEITZAHL_ZEIT_s
	subi temp1, BYTE1   (-60)
	sbci temp2, BYTE2   (-60)

	ldi temp,100
	mul temp,temp9
	add temp1,R0
	adc temp2,R1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Bsp 02:00 ---> 120															
ZEIT_ZEITZAHL:
	ldi temp9,(-1)
ZEITZAHL_s:
	inc temp9
	subi temp1, BYTE1    (100)
	sbci temp2, BYTE2    (100)
	brcc ZEITZAHL_s
	subi temp1, BYTE1   (-100)
	sbci temp2, BYTE2   (-100)

	ldi temp,60
	mul temp,temp9
	add temp1,R0
	adc temp2,R1
;-------------------------------------------------------------------------------
	cpi temp2,high(24*60-1)
	brlo ZEITZAHL_EXIT
	breq ZEITZAHL_w
	brsh ZEITZAHL_ERROR
ZEITZAHL_w:
	cpi temp1,low (24*60-1)
	brlo ZEITZAHL_EXIT
	breq ZEITZAHL_EXIT
	brsh ZEITZAHL_ERROR
;-------------------------------------------------------------------------------
ZEITZAHL_EXIT:
	ret
;-------------------------------------------------------------------------------
ZEITZAHL_ERROR:
	ldi temp1,low (24*60-1)
	ldi temp2,high(24*60-1)
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; temp1: 23 Tag																	
; temp2: 12 Monat																
; temp3: 19 Jahr																
; OUT:  "231219"																
FUNCTION_SPEZIAL_TEMP1_2_3:
	STS(adr_DUMMY0),temp1
	STS(adr_DUMMY1),temp2
	STS(adr_DUMMY2),temp3
	call TEMP1_2_3_4_NULL		
	LDS temp1,(adr_DUMMY0)			; Tag	
	call TEMP1_2_3_4_MAL_1000
	call TEMP1_2_3_4_MAL_10
	LDS temp,(adr_DUMMY1)		; 
	ldi temp9,100
	mul temp,temp9
	add temp1,R0
	adc temp2,R1
	adc temp3,NULL
	adc temp4,NULL
	LDS temp,(adr_DUMMY2)
	add temp1,temp
	adc temp2,NULL
	adc temp3,NULL
	adc temp4,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; INP:  "231219"																
; OUT_temp1: 23 Tag																
; OUT_temp2: 12 Monat															
; OUT_temp3: 19 Jahr															
FUNCTION_REVERS_TEMP1_2_3:
	ldi temp5,(-1)
FUNCTION_REVERS_a:		; 10.000	
	inc temp5
	subi temp1, BYTE1(10000)
	sbci temp2, BYTE2(10000)
	sbci temp3, BYTE3(10000)
	brcc FUNCTION_REVERS_a
	subi temp1, BYTE1(-10000)
	sbci temp2, BYTE2(-10000)
	sbci temp3, BYTE3(-10000)

	ldi temp6,(-1)
FUNCTION_REVERS_b:		; 100	
	inc temp6
	subi temp1, BYTE1(100)
	sbci temp2, BYTE2(100)
	sbci temp3, BYTE3(100)
	brcc FUNCTION_REVERS_b
	subi temp1, BYTE1(-100)
	sbci temp2, BYTE2(-100)
	sbci temp3, BYTE3(-100)

	mov temp3,temp1
	mov temp2,temp6
	mov temp1,temp5
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZUFALLSZAHL_TEMP:
	clr ZL	
	clr ZH
	ldi temp9,255
ZUFALLSZAHL_s:
	LD R0,Z+
	add temp,R0
	dec temp9
	brne ZUFALLSZAHL_s
	STS(adr_ZUFALLSZAHL),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZAHLER_NULL:
	STS(adr_ZAEHLER0),NULL
	STS(adr_ZAEHLER1),NULL
	STS(adr_ZAEHLER2),NULL
	STS(adr_ZAEHLER3),NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZAHLER_PLUS_EINS:
	LDS temp1,(adr_ZAEHLER0)
	LDS temp2,(adr_ZAEHLER1)
	LDS temp3,(adr_ZAEHLER2)
	LDS temp4,(adr_ZAEHLER3)
	add temp1,EINS
	adc temp2,NULL
	adc temp3,NULL
	adc temp4,NULL
	STS(adr_ZAEHLER0),temp1
	STS(adr_ZAEHLER1),temp2
	STS(adr_ZAEHLER2),temp3
	STS(adr_ZAEHLER3),temp4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_NULL:
	clr SEKTOR0
	clr SEKTOR1
	clr SEKTOR2
	clr SEKTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_PLUS_EINS:
	add SEKTOR0,EINS
	adc SEKTOR1,NULL
	adc SEKTOR2,NULL
	adc SEKTOR3,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_MINUS_EINS:
	tst SEKTOR0
	brne SEKTOR_MINUS_RUN
	tst SEKTOR1
	brne SEKTOR_MINUS_RUN
	tst SEKTOR2
	brne SEKTOR_MINUS_RUN
	tst SEKTOR3
	brne SEKTOR_MINUS_RUN
	ret
;-------------------------------------------------------------------------------
SEKTOR_MINUS_RUN:
	sub SEKTOR0,EINS
	sbc SEKTOR1,NULL
	sbc SEKTOR2,NULL
	sbc SEKTOR3,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_MAL_2:
	LSL SEKTOR0					; x2	
	ROL SEKTOR1
	ROL SEKTOR2
	ROL SEKTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_DURCH_2:
	LSR SEKTOR3					; :2	
	ROR SEKTOR2
	ROR SEKTOR1
	ROR SEKTOR0
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_4_NACH_SEKTOR:
	MOV SEKTOR0,temp1
	MOV SEKTOR1,temp2
	MOV SEKTOR2,temp3
	MOV SEKTOR3,temp4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SEKTOR_NACH_TEMP1_4:
	MOV temp1,SEKTOR0
	MOV temp2,SEKTOR1
	MOV temp3,SEKTOR2
	MOV temp4,SEKTOR3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_NULL:
	clr temp1
	clr temp2
	clr temp3
	clr temp4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_PLUS_EINS:
	add temp1,EINS						; +1		
	adc temp2,NULL
	adc temp3,NULL
	adc temp4,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MINUS_EINS:
	sub temp1,EINS						; -1		
	sbc temp2,NULL
	sbc temp3,NULL
	sbc temp4,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_2:
	LSR TEMP4					; :2	
	ROR TEMP3
	ROR TEMP2
	ROR TEMP1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_2:
	LSL TEMP1					; x2	
	ROL TEMP2
	ROL TEMP3
	ROL TEMP4
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_512:
	mov temp4,temp3			; 8xlinks		
	mov temp3,temp2
	mov temp2,temp1
	clr temp1				; LOW			

	LSL temp1				; LOW 1xlinks	
	ROL temp2
	ROL temp3
	ROL temp4				; HIGH			
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_10:
	mov R0,temp1
	mov R1,temp2
	mov R2,temp3
	mov R3,temp4
	ldi temp9,9
;-------------------------------------------------------------------------------
TEMP1_2_3_4_MAL_10_s:
	add temp1,R0	
	adc temp2,R1
	adc temp3,R2
	adc temp4,R3
	dec temp9
	brne TEMP1_2_3_4_MAL_10_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_MAL_1000:
	clr R0
	clr R1
	clr R2
	clr R3
	ldi ZL,low (1000)
	ldi ZH,high(1000)
TEMP1_2_3_4_MAL_1000_s:
	add R0,temp1
	adc R1,temp2
	adc R2,temp3
	adc R3,temp4
	sbiw ZL,1		; -1
	tst ZL
	brne TEMP1_2_3_4_MAL_1000_s
	tst ZH
	brne TEMP1_2_3_4_MAL_1000_s
	mov temp1,R0
	mov temp2,R1
	mov temp3,R2
	mov temp4,R3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_10:
	ldi temp5,LOW (10)
	ldi temp6,HIGH(10)
	rjmp DIVISION_32_16_NO_RUND
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_512:
	mov temp1,temp2			; 8xlinks		
	mov temp2,temp3
	mov temp3,temp4
	clr temp4
	LSR TEMP4				; :2	
	ROR TEMP3
	ROR TEMP2
	ROR TEMP1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEMP1_2_3_4_DURCH_1000:
	ldi temp5,LOW (1000)
	ldi temp6,HIGH(1000)
	rjmp DIVISION_32_16
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version: 14.11.2014																	
;																						
; 	INPUT:	TEMP1 / TEMP2 	1.Faktor													
;			TEMP3 / TEMP4	2.Faktor													
; 																						
; 	OUTPUT: TEMP1...4	Produkt															
; 																						
;	HILFSREGISTER: 	temp5+6+7+8 (werden gesichert)										
; 																						
MULTIPLIKATION_16_16_32:
	push temp5
	push temp6
	push temp7
	push temp8

	clr	r2
	mul	temp4,temp2		; ah * bh
	mov	temp7,r0
	mov	temp8,r1
	mul	temp3,temp1		; al * bl
	mov	temp5,r0
	mov	temp6,r1
	mul	temp4,temp1		; ah * bl
	add	temp6,r0
	adc	temp7,r1
	adc	temp8,r2
	mul	temp2,temp3		; bh * al
	add	temp6,r0
	adc	temp7,r1
	adc	temp8,r2


	mov temp1,temp5
	mov temp2,temp6
	mov temp3,temp7
	mov temp4,temp8

	pop temp8
	pop temp7
	pop temp6
	pop temp5

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Version: 11/2014																
;																				
;	Division 32 bit-Zahl durch 16 bit Zahl										
;																				
;	input/Output 	temp1 (LOW)	(0...4.294.967.295)								
;	input/Output 	temp2														
;	input/Output 	temp3														
;	input/Output 	temp4 (HIGH)												
;																				
;	input	temp5	Divisor	(LOW)	 											
;	input	temp6	Divisor	(HIGH)	 											
;																				
;	 		temp	Hilfsregister 												
;																				
DIVISION_32_16:
	; Fehlerprfung		
	tst temp5
	brne DIVISION_32_16_RUN
	tst temp6
	brne DIVISION_32_16_RUN
	; Fehler NULL		
	clr temp1
	clr temp2
	clr temp3
	clr temp4
	ret	
;-------------------------------------------------------------------------------
DIVISION_32_16_RUN:
	; RUNDEN halben Divisor dazuaddieren
	mov R0,temp5		; KOPIE			
	mov R1,temp6
	LSR R1				; durch 2		
	ROR R0
	add temp1,R0		; addieren		
	adc temp2,R1
	adc temp3,NULL
	adc temp4,NULL
;-------------------------------------------------------------------------------
DIVISION_32_16_NO_RUND:
	ldi     temp,32    ;32* schieben	
    clr     R0          ;Rest =0		
    clr     R1
DIVISION_32_16_1:    
	lsl     temp1       ;Ein Bit aus dem Dividend ...   
    rol     temp2       ;ins Carry schieben
    rol     temp3
    rol     temp4
    rol     R0          ;Carry in den Rest schieben
    rol     R1
    brcs    DIVISION_32_16_3       ;Bei berlauf von Rest, Sprung
    cp      R0,temp5       					;Solange Bits in temp1 (Rest) schieben bis >= divisor
    cpc     R1,temp6
    brcs    DIVISION_32_16_2    	;Bei Rest<Divisor, die eingeschobene 0 in temp1 stehen lassen
DIVISION_32_16_3:
    inc     temp1       ;Ansonsten Bit0 setzen und den Divisor vom Rest abziehen
    sub     R0,temp5
    sbc     R1,temp6
DIVISION_32_16_2:
   	dec     temp       ;Das ganze 32 mal
    brne    DIVISION_32_16_1
ret 
; ##############################################################################
; ##############################################################################
; ############################################################################## 
; ##############################################################################
; ##############################################################################
; ##############################################################################






