; ##############################################################################
; ##############################################################################
; ##############################################################################
; SPCR => SPIE SPE DORD MSTR CPOL CPHA SPR1 SPR0 (SPI Control Register)			
; 																				
; SPI Mode Conditions Leading Edge Trailing eDge								
; 0 CPOL=0, CPHA=0 Sample (Rising) 	Setup (Falling)								
; 1 CPOL=0, CPHA=1 Setup (Rising) 	Sample (Falling)							
; 2 CPOL=1, CPHA=0 Sample (Falling) Setup (Rising)								
; 3 CPOL=1, CPHA=1 Setup (Falling) 	Sample (Rising)								
; 																				
; SPI2X SPR1 SPR0 	SCK Frequency												
; 0 	0 	 0 		fosc/4														
; 0 	0 	 1 		fosc/16														
; 0 	1 	 0 		fosc/64														
; 0 	1 	 1 		fosc/128													
; 1 	0 	 0 		fosc/2														
; 1 	0 	 1 		fosc/8														
; 1 	1 	 0 		fosc/32														
; 1 	1 	 1 		fosc/64														
; 																				
SPI_INITIALISIERUNG:
; SPI-CS										
.equ SPI_CS_PIN_NR			= 4
.equ SPI_CS_DDR				= DDRB
.equ SPI_CS_PORT			= PORTB
; SPI-MOSI										
.equ SPI_MOSI_PIN_NR		= 5
.equ SPI_MOSI_DDR			= DDRB
.equ SPI_MOSI_PORT			= PORTB
; SPI-MISO										
.equ SPI_MISO_PIN_NR		= 6
.equ SPI_MISO_DDR			= DDRB
.equ SPI_MISO_PORT			= PORTB
; SPI-SCK										
.equ SPI_SCK_PIN_NR			= 7
.equ SPI_SCK_DDR			= DDRB
.equ SPI_SCK_PORT			= PORTB


	rcall SPI_CS_HIGH				 		; HIGH (LOW=aktiv)
	; PORTS									
	sbi (SPI_CS_DDR)  ,(SPI_CS_PIN_NR)		; PORT auf Ausgang	
	sbi (SPI_MOSI_DDR),(SPI_MOSI_PIN_NR)	; PORT auf Ausgang	
	sbi (SPI_SCK_DDR) ,(SPI_SCK_PIN_NR)		; PORT auf Ausgang	
	cbi (SPI_MISO_DDR),(SPI_MISO_PIN_NR)	; PORT auf EINGANG	
	sbi (SPI_MISO_PORT),(SPI_MISO_PIN_NR)	; Pull-UP on		
	call wait_200ms							; diese PAUSE ist wichtig		
	call SPI_TAKT_LOW
	call SPI_CHECK
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_CHECK:
	; immer Ausgang					
	sbis (SPI_CS_DDR),(SPI_CS_PIN_NR)		; springe, wenn ...	
	rjmp SPI_CHECK_ERROR
	sbis (SPI_MOSI_DDR),(SPI_MOSI_PIN_NR)	; springe, wenn ...	
	rjmp SPI_CHECK_ERROR
	sbis (SPI_SCK_DDR) ,(SPI_SCK_PIN_NR)	; springe, wenn ...	
	rjmp SPI_CHECK_ERROR
	; immer EINGANG					
	sbic (SPI_MISO_DDR),(SPI_MISO_PIN_NR)	; springe, wenn ...	
	rjmp SPI_CHECK_ERROR
	; immer Pull-Up on				
	sbis (SPI_MISO_PORT),(SPI_MISO_PIN_NR)	; springe, wenn ...	
	rjmp SPI_CHECK_ERROR
	ret
; ------------------------------------------------------------------------------
SPI_CHECK_ERROR:

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; SPI2X SPR1 SPR0 	SCK Frequency												
; 0 	0 	 0 		fosc/4														
; 0 	0 	 1 		fosc/16														
; 0 	1 	 0 		fosc/64														
; 0 	1 	 1 		fosc/128													
; 1 	0 	 0 		fosc/2														
; 1 	0 	 1 		fosc/8														
; 1 	1 	 0 		fosc/32														
; 1 	1 	 1 		fosc/64														
; 																				
; fosc/128																		
SPI_TAKT_LOW:
	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(1<<SPR1)|(1<<SPR0); /128
;	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(1<<SPR1)|(0<<SPR0); /64 
;	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(0<<SPR1)|(1<<SPR0); /16 
	out SPCR,temp
	ldi temp,(0<<SPI2X)	; SPIF WCOL      SPI2X
	out SPSR,temp
ret
; ##############################################################################
SPI_TAKT_HIGH:
	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(0<<SPR1)|(0<<SPR0)
	out SPCR,temp
	ldi temp,(0<<SPI2X)	; SPIF WCOL      SPI2X
	out SPSR,temp
ret
; ##############################################################################
; fosc/2																		
SPI_TAKT_SUPER_HIGH:
	ldi temp,(0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(0<<CPHA)|(0<<SPR1)|(0<<SPR0)
	out SPCR,temp
	ldi temp,(1<<SPI2X)	; SPIF WCOL      SPI2X
	out SPSR,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; inaktiv																		
SPI_CS_HIGH:
	sbi(SPI_CS_PORT),(SPI_CS_PIN_NR)		; CS auf HIGH	
	LED_GELB2_AUS
	WDR
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; aktiv																			
SPI_CS_LOW:
	cbi  (SPI_CS_PORT),(SPI_CS_PIN_NR)		; CS auf LOW !!!	
	LED_GELB2_AN
	;call WAIT_1us							; ev. kleine PAUSE	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; inp: TEMP																		
; OUT: TEMP																		
SPI_TX_RX_TEMP:
	out SPDR,temp			; Start transmission of data 	
SPI_TX_RX_TEMP_w:
	in temp,SPSR
	sbrs temp,SPIF
	rjmp SPI_TX_RX_TEMP_w	; Wait for transmission complete
	in temp, SPDR			; DATENBYTE EINLESEN			
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; TX:  0xFF	(DUMMY)																
; OUT: temp																		
SPI_RX_TEMP:
	out SPDR,VOLL			; Start transmission of data 	; DUMMY (xFF)					
SPI_RX_TEMP_w:
	in temp,SPSR
	sbrs temp,SPIF
	rjmp SPI_RX_TEMP_w		; Wait for transmission complete
	in temp, SPDR			; DATENBYTE EINLESEN			
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 512 Bytes SD--->SRAM															
SPI_RX512_DATA:
	push ZL						; sichen	
	push ZH
	ldi ZL, LOW(adr_SD_DATA)	; Z-ZEIGER initialisieren	
	ldi ZH,HIGH(adr_SD_DATA)
	ldi temp9,32				; Schleife (32x16=512)		
SPI_RX512_DATA_s:
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	rcall SPI_RX_TEMP			; OUT: temp
	ST Z+,temp					; speichern und Z+1	
	dec temp9
	brne SPI_RX512_DATA_s
	pop ZH						; wieder herstellen	
	pop ZL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 512 Bytes SRAM--->SD															
SPI_TX512_DATA:
	push ZL							; sichen	
	push ZH
	ldi ZL, LOW(adr_SD_DATA)			; Z-ZEIGER initialisieren	
	ldi ZH,HIGH(adr_SD_DATA)
	ldi temp9,32					; Schleife (512)		
SD_SEKTOR_WRITE_DATA:
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	LD temp,Z+						; DATA				
	rcall SPI_TX_RX_TEMP			; INP/OUT: TEMP		
	dec  temp9						; Schleife			
	brne SD_SEKTOR_WRITE_DATA
	pop ZH							; wieder herstellen	
	pop ZL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; max 1s																		
; auf  RX=temp warten															
; OUT: Carry																	
SPI_WAIT_TEMP1_C:
	push ZL					; sichen	
	push ZH
	ldi ZL,low (65535)
	ldi ZH,high(65535)
; ------------------------------------------------------------------------------
SPI_WAIT_s:
	WDR
	rcall SPI_RX_TEMP		; OUT: temp	
	cp temp,temp1			; Vergleich	
	breq SPI_WAIT_OK
	sbiw ZL,1				; -1		
	tst ZL
	brne SPI_WAIT_s	; Schleife	
	tst ZH
	brne SPI_WAIT_s	; Schleife	
; ------------------------------------------------------------------------------
SPI_WAIT_ERROR:
	LED_ROT2_AN
	pop ZH					; wieder herstellen	
	pop ZL
	CLC 					; Clear Carry	
	ret
; ------------------------------------------------------------------------------
SPI_WAIT_OK:
	pop ZH					; wieder herstellen	
	pop ZL
	SEC 					; Set Carry		
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; auf  3x hintereinander 0xFF warten, dann CS_HIGH								
SPI_WAIT_FF_HIGH:
	rcall SPI_RX_TEMP		; OUT: temp
	cpi temp,0xFF
	brne SPI_WAIT_FF_HIGH
	rcall SPI_RX_TEMP		; OUT: temp
	cpi temp,0xFF
	brne SPI_WAIT_FF_HIGH
	rcall SPI_RX_TEMP		; OUT: temp
	cpi temp,0xFF
	brne SPI_WAIT_FF_HIGH
	rjmp SPI_CS_HIGH		; CS auf HIGH		
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_CLEAR_HIGH:
	rcall SPI_CLEAR255
	rcall SPI_CLEAR255
	rcall SPI_CLEAR255
	rcall SPI_CLEAR255
	rcall SPI_CS_HIGH		; CS auf HIGH		
	rcall SPI_CLEAR255
ret
; ------------------------------------------------------------------------------
SPI_CLEAR255:
	push temp9
	ldi temp9,255
SPI_CLEAR_s:
	rcall SPI_RX_TEMP			; OUT: temp
	dec temp9
	brne SPI_CLEAR_s
	pop temp9
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
SPI_DEBUG_ANZEIGE:

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################


