; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 12/2015															   
TWI_INI:
.equ TWI_BIT_RATE							= 255		; 0..255  Bit-Rate					
.equ TWI_PRESCALER							= 1			; 0...3 (0=1 / 1=4 / 2=16 / 3=64)	
.equ TWI_SCL_BERECHNET						= TAKT/(16+(2*TWI_BIT_RATE)*(exp2(TWI_PRESCALER))*(exp2(TWI_PRESCALER)))
.equ TWI_SCL_PIN_NR							= 0
.equ TWI_SCL_PORT							= PORTC
.equ TWI_SCL_PIN							= PINC
.equ TWI_SDA_PIN_NR							= 1
.equ TWI_SDA_PORT							= PORTC
.equ TWI_SDA_PIN							= PINC
	; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)	
	; Beispiel:										
	; 674 Hz = 22.000.000 / (16 + 2 * 255 * 4^3)	

	; TWI TWBR 				Bit RATE Register	
	ldi temp, (TWI_BIT_RATE);0..255				
	STS TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, (TWI_PRESCALER)
	andi temp,0b00000011	; 0...3 zulssig	
	STS TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	STS TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	STS TWCR,temp
	; PULL-UP ein (ca. 30k)						
	sbi (TWI_SCL_PORT),(TWI_SCL_PIN_NR)
	sbi (TWI_SDA_PORT),(TWI_SDA_PIN_NR)
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 12/2015															   
; Anzeige, wenn SDA oder SCL gegen GND gezogen wird							   
TWI_BUS_FREE_CHECK:
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 12/2015															   
TWI_AUS:
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	STS TWCR,temp
	call wait_100ms
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 12/2015															   
;																			   
; MT-START-ROUTINE	(MASTER TRANSMITTER)									   
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
; OUTPUT: 	ERROR (FEHLER_CODE)									   
;																			   
TWI_MT_START_TEMP1:
	WDR
	clr  ERROR 					; CLEAR							
	; START								
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	STS TWCR, temp	
	rcall TWI_TWINT_WAIT					; warten, bis TWINT wieder = 1	
	; Check TWSR						
	LDS temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08							; 0x08 START erfolgreich		
	breq TWI_MT_START_w
	cpi temp,  0x10							; 0x10 (repeated START)			
	breq TWI_MT_START_w
	rjmp TWI_MT_START_ERROR					; ERROR							
;-------------------------------------------------------------------------------
TWI_MT_START_w:
	; Load SLA + "W"  (W=0) 			
	mov temp,temp1							; KOPIE							
	LSL  temp								; 1 x links						
	STS TWDR,temp
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	STS TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_WAIT									
	; Check TWSR						
	LDS temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18							; 0x18							
	brne TWI_MT_START_ERROR					; ERROR							
	ret
;-------------------------------------------------------------------------------
; ERROR																			
TWI_MT_START_ERROR:
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  an	
	ldi temp,(ERROR_TWI_MT_START)
	mov ERROR, temp  				;ERROR				
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 12/2015															   
; 																			   
; Ein BYTE zum SLAVE zu senden										   	   	   
; 																			   
; INPUT:	temp1 <= DATEN													   
; OUT: 		ERROR (FEHLER_CODE)   								   			   
; 																			   
TWI_MT_DATA_TEMP1:
	WDR
	tst  ERROR 							; ERROR=0 ?		
	breq TWI_MT_DATA_RUN
	ret										
;-------------------------------------------------------------------------------
TWI_MT_DATA_RUN:
	; 	Load DATA						
	STS TWDR, temp1							
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	STS TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_WAIT
	; Check TWSR						
	LDS temp,TWSR
	andi temp,0xF8									; Bitmuster		
	cpi temp, 0x28									; 0x28			
	brne TWI_MT_DATA_ERROR							; ERROR			
	ret
;-------------------------------------------------------------------------------
; ERROR																			
TWI_MT_DATA_ERROR:
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  an	
	ldi temp,(ERROR_TWI_MT_DATA)
	mov ERROR, temp 						; ERROR		
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 12/2015															   
;																			   
; MR-START-ROUTINE													   		   
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
; OUT: 		ERROR															   
;																			   
TWI_MR_START_TEMP1:
	WDR
	clr ERROR 							; CLEAR					
	; SRAM lschen						
	STS	(adr_TWI_BEREICH_READ+0),NULL
	STS	(adr_TWI_BEREICH_READ+1),NULL
	STS	(adr_TWI_BEREICH_READ+2),NULL
	STS	(adr_TWI_BEREICH_READ+3),NULL
	STS	(adr_TWI_BEREICH_READ+4),NULL
	STS	(adr_TWI_BEREICH_READ+5),NULL
	STS	(adr_TWI_BEREICH_READ+6),NULL
	; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	STS TWCR, temp	
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_WAIT									
	; Check TWSR 										
	LDS temp,TWSR		
	andi temp, 0xF8									; Bitmuster				
	cpi temp,  0x08 								; 0x08 START erfolgreich
	breq TWI_MR_START_RUN
	cpi temp,  0x10									; 0x10 (repeated START)	
	breq TWI_MR_START_RUN
	rjmp TWI_MR_START_ERROR
;-------------------------------------------------------------------------------
TWI_MR_START_RUN:
	; Load SLA +"R" (R=1)								
	mov temp,temp1									; KOPIE			
	LSL temp										; 1 x links		
	ori temp, 0b00000001 							; BIT setzen	
	STS TWDR, temp
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_WAIT									
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8									; Bitmuster		
	cpi temp,  0x40 								; SLA+R gesendet; ACK empfangen
	brne TWI_MR_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MR_START_ERROR:
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  an	
	ldi temp,(ERROR_TWI_MR_START)
	mov ERROR, temp 						;ERROR			
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 08/2015															   	
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! BYTE empfangen werden sollen											
;																				
; OUTPUT: temp1																	
; OUTPUT: ERROR(FEHLER_CODE)	   												
;																				
TWI_MR_DATA_TEMP1:
	WDR
	tst  ERROR 						; ERROR=0 ?				
	breq TWI_MR_DATA_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MR_DATA_RUN:
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_WAIT									
	; Check TWSR										
	LDS temp,TWSR
	andi temp,0xF8								; Bitmuster				
	cpi temp, 0x50								; Data Byte empfangen- ACK zurckgesendet
	brne TWI_MR_DATA_ERROR
	; DATEN einlesen									
	LDS temp1, TWDR	
	ret
;-------------------------------------------------------------------------------
; ERROR																			
TWI_MR_DATA_ERROR:
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  an	
	ldi temp,(ERROR_TWI_MR_DATA)
	mov ERROR, temp 					;ERROR		
	clr temp1				
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 12/2015															   
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! BYTE empfangen werden soll, oder wenn								
; das letzte! BYTE empfangen werden soll										
;																				
; OUTPUT: temp1																	
; OUTPUT: ERROR(FEHLER_CODE)	   										
;																				
TWI_MR_LAST_DATA_TEMP1:
	tst  ERROR 									; ERROR=0 ?				
	breq TWI_MR_LAST_DATA_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MR_LAST_DATA_RUN:
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	STS TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_WAIT		
	; Check TWSR										
	LDS temp,TWSR
	andi temp, 0xF8								; Bitmuster				
	cpi  temp, 0x58								; letztes Data Byte empfangen-NOT ACK zurckgesendet
	brne TWI_MR_LAST_DATA_ERROR
	; DATEN einlesen									
	LDS temp1, TWDR
	ret
;-------------------------------------------------------------------------------
; ERROR																			
TWI_MR_LAST_DATA_ERROR:
	cbi (LED_ROT_PORT)  ,(LED_ROT_PIN_NR)		; ROT  an	
	ldi temp,(ERROR_TWI_MR_DATAEND)
	mov ERROR, temp 					; ERROR		
	clr temp1
ret
; #############################################################################
; #############################################################################
; #############################################################################
; VERSION: 12/2015															   
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	STS TWCR, temp	
	; kurze Pause
	call wait_1us
ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_TWINT_WAIT:
	WDR
	LDS temp, TWCR		; TWCR auslesen	
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG gesetzt ist
	rjmp TWI_TWINT_WAIT
	ret
; #############################################################################
; #############################################################################
; #############################################################################


